from __future__ import absolute_import
import logging
import mock
import unittest
from ZConfig import configureLoggers


class TestConfig(unittest.TestCase):

    @mock.patch("raven.handlers.logging.Client")
    def test_minimal(self, Client):
        configureLoggers("""
        <logger>
          %import raven.contrib.zconfig
          <sentry>
            dsn https://abc:def@example.com/42
          </sentry>
        </logger>
        """)
        handler = logging.getLogger().handlers[-1]
        logging.getLogger().handlers.remove(handler)
        self.assertEqual(handler.level, logging.ERROR)
        Client.assert_called_with(
            dsn='https://abc:def@example.com/42',
            site=None,
            name=None,
            release=None,
            environment=None,
            exclude_paths=None,
            include_paths=None,
            sample_rate=1.0,
            list_max_length=None,
            string_max_length=None,
            auto_log_stacks=None,
            processors=None,
            level=logging.ERROR)

    @mock.patch("raven.handlers.logging.Client")
    def test_many(self, Client):
        configureLoggers("""
        <logger>
          %import raven.contrib.zconfig
          <sentry>
            level WARNING
            dsn https://abc:def@example.com/42
            site test-site
            name test
            release 42.0
            environment testing
            exclude_paths /a /b
            include_paths /c /d
            sample_rate 0.5
            list_max_length 9
            string_max_length 99
            auto_log_stacks true
            processors x y z
          </sentry>
        </logger>
        """)
        handler = logging.getLogger().handlers[-1]
        logging.getLogger().handlers.remove(handler)
        self.assertEqual(handler.level, logging.WARNING)
        Client.assert_called_with(
            dsn='https://abc:def@example.com/42',
            site='test-site',
            name='test',
            release='42.0',
            environment='testing',
            exclude_paths=['/a', '/b'],
            include_paths=['/c', '/d'],
            sample_rate=0.5,
            list_max_length=9,
            string_max_length=99,
            auto_log_stacks=True,
            processors=['x', 'y', 'z'],
            level=logging.WARNING)



import json
import logging
import pytest
import sys

from exam import before, fixture
from mock import patch, Mock

from raven.contrib.sanic import Sentry, logging_configured
from raven.handlers.logging import SentryHandler
from raven.utils.testutils import InMemoryClient, TestCase


if sys.version_info >= (3, 5):
    from sanic import Sanic, response

# When using test_client, Sanic will run the server at 127.0.0.1:42101.
# For ease of reading, let's establish that as a constant.
BASE_URL = '127.0.0.1:42101'

def create_app(ignore_exceptions=None, debug=False, **config):
    import os
    app = Sanic(__name__)
    for key, value in config.items():
        app.config[key] = value

    app.debug = debug

    if ignore_exceptions:
        app.config['RAVEN_IGNORE_EXCEPTIONS'] = ignore_exceptions

    @app.route('/an-error/', methods=['GET', 'POST'])
    def an_error(request):
        raise ValueError('hello world')

    @app.route('/log-an-error/', methods=['GET'])
    def log_an_error(request):
        logger = logging.getLogger('random-logger')
        logger.error('Log an error')
        return response.text('Hello')

    @app.route('/capture/', methods=['GET', 'POST'])
    def capture_exception(request):
        try:
            raise ValueError('Boom')
        except Exception:
            request.app.extensions['sentry'].captureException()
        return response.text('Hello')

    @app.route('/message/', methods=['GET', 'POST'])
    def capture_message(request):
        request.app.extensions['sentry'].captureMessage('Interesting')
        return response.text('World')

    return app

@pytest.mark.skipif(sys.version_info < (3,5), reason="Requires Python 3.5+.")
class BaseTest(TestCase):
    @fixture
    def app(self):
        return create_app()

    @fixture
    def client(self):
        return self.app.test_client

    @before
    def bind_sentry(self):
        self.raven = InMemoryClient()
        self.middleware = Sentry(self.app, client=self.raven)

    def make_client_and_raven(self, logging=False, *args, **kwargs):
        app = create_app(*args, **kwargs)
        raven = InMemoryClient()
        Sentry(app, logging=logging, client=raven)
        return app.test_client, raven, app

@pytest.mark.skipif(sys.version_info < (3,5), reason="Requires Python 3.5+.")
class SanicTest(BaseTest):
    def test_does_add_to_extensions(self):
        self.assertIn('sentry', self.app.extensions)
        self.assertEquals(self.app.extensions['sentry'], self.middleware)

    def test_error_handler(self):
        request, response = self.client.get('/an-error/')
        self.assertEquals(response.status, 500)
        self.assertEquals(len(self.raven.events), 1)

        event = self.raven.events.pop(0)

        assert 'exception' in event
        exc = event['exception']['values'][-1]
        self.assertEquals(exc['type'], 'ValueError')
        self.assertEquals(exc['value'], 'hello world')
        self.assertEquals(event['level'], logging.ERROR)
        self.assertEquals(event['message'], 'ValueError: hello world')

    def test_capture_plus_logging(self):
        client, raven, app = self.make_client_and_raven(
            debug=False, logging=True)
        client.get('/an-error/')
        client.get('/log-an-error/')
        assert len(raven.events) == 2

    def test_get(self):
        request, response = self.client.get('/an-error/?foo=bar')
        self.assertEquals(response.status, 500)
        self.assertEquals(len(self.raven.events), 1)

        event = self.raven.events.pop(0)

        assert 'request' in event
        http = event['request']
        self.assertEquals(http['url'], 'http://{0}/an-error/'.format(BASE_URL))
        self.assertEquals(http['query_string'], 'foo=bar')
        self.assertEquals(http['method'], 'GET')
        self.assertEquals(http['data'], {})
        self.assertTrue('headers' in http)
        headers = http['headers']
        self.assertTrue('host' in headers, headers.keys())
        self.assertEqual(headers['host'], BASE_URL)
        self.assertTrue('user-agent' in headers, headers.keys())
        self.assertTrue('aiohttp' in headers['user-agent'])

    def test_post_form(self):
        request, response = self.client.post('/an-error/?biz=baz', data={'foo': 'bar'})
        self.assertEquals(response.status, 500)
        self.assertEquals(len(self.raven.events), 1)
        event = self.raven.events.pop(0)

        assert 'request' in event
        http = event['request']
        self.assertEquals(http['url'], 'http://{0}/an-error/'.format(BASE_URL))
        self.assertEquals(http['query_string'], 'biz=baz')
        self.assertEquals(http['method'], 'POST')
        self.assertEquals(http['data'], {'foo': ['bar']})
        self.assertTrue('headers' in http)
        headers = http['headers']
        self.assertTrue('host' in headers, headers.keys())
        self.assertEqual(headers['host'], BASE_URL)
        self.assertTrue('user-agent' in headers, headers.keys())
        self.assertTrue('aiohttp' in headers['user-agent'])

    def test_post_json(self):
        request, response = self.client.post(
            '/an-error/?biz=baz', data=json.dumps({'foo': 'bar'}),
            headers={'content-type': 'application/json'})
        self.assertEquals(response.status, 500)
        self.assertEquals(len(self.raven.events), 1)
        event = self.raven.events.pop(0)
        assert 'request' in event
        http = event['request']
        self.assertEquals(http['url'], 'http://{0}/an-error/'.format(BASE_URL))
        self.assertEquals(http['query_string'], 'biz=baz')
        self.assertEquals(http['method'], 'POST')
        self.assertEquals(http['data'], {'foo': 'bar'})
        self.assertTrue('headers' in http)
        headers = http['headers']
        self.assertTrue('host' in headers, headers.keys())
        self.assertEqual(headers['host'], BASE_URL)
        self.assertTrue('user-agent' in headers, headers.keys())
        self.assertTrue('aiohttp' in headers['user-agent'])

    def test_captureException_captures_http(self):
        request, response = self.client.get('/capture/?foo=bar')
        self.assertEquals(response.status, 200)
        self.assertEquals(len(self.raven.events), 1)

        event = self.raven.events.pop(0)
        self.assertEquals(event['event_id'], response.headers['X-Sentry-ID'])

        assert event['message'] == 'ValueError: Boom'
        print(event)
        assert 'request' in event
        assert 'exception' in event

    def test_captureMessage_captures_http(self):
        request, response = self.client.get('/message/?foo=bar')
        self.assertEquals(response.status, 200)
        self.assertEquals(len(self.raven.events), 1)

        event = self.raven.events.pop(0)
        self.assertEquals(event['event_id'], response.headers['X-Sentry-ID'])

        assert 'sentry.interfaces.Message' in event
        assert 'request' in event

    def test_captureException_sets_last_event_id(self):
        try:
            raise ValueError
        except Exception:
            self.middleware.captureException()
        else:
            self.fail()

        event_id = self.raven.events.pop(0)['event_id']
        assert self.middleware.last_event_id == event_id

    def test_captureMessage_sets_last_event_id(self):
        self.middleware.captureMessage('foo')

        event_id = self.raven.events.pop(0)['event_id']
        assert self.middleware.last_event_id == event_id

    def test_logging_setup_signal(self):
        app = Sanic(__name__)

        mock_handler = Mock()

        def receiver(sender, *args, **kwargs):
            self.assertIn("exclude", kwargs)
            mock_handler(*args, **kwargs)

        logging_configured.connect(receiver)
        raven = InMemoryClient()

        Sentry(
            app, client=raven, logging=True,
            logging_exclusions=("excluded_logger",))

        mock_handler.assert_called()

    def test_check_client_type(self):
        self.assertRaises(TypeError, lambda _: Sentry(self.app, "oops, I'm putting my DSN instead"))

    def test_uses_dsn(self):
        app = Sanic(__name__)
        sentry = Sentry(app, dsn='http://public:secret@example.com/1')
        assert sentry.client.remote.base_url == 'http://example.com'

    def test_binds_default_include_paths(self):
        app = Sanic(__name__)
        sentry = Sentry(app, dsn='http://public:secret@example.com/1')
        assert sentry.client.include_paths == set([app.name])

    def test_overrides_default_include_paths(self):
        app = Sanic(__name__)
        app.config['SENTRY_CONFIG'] = {'include_paths': ['foo.bar']}
        sentry = Sentry(app, dsn='http://public:secret@example.com/1')
        assert sentry.client.include_paths == set(['foo.bar'])



"""
raven.utils.http
~~~~~~~~~~~~~~~~

:copyright: (c) 2010-2012 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import

import socket
import ssl
import sys

from raven.conf import defaults
from raven.utils.compat import urllib2, httplib
from raven.utils.ssl_match_hostname import match_hostname


def urlopen(url, data=None, timeout=defaults.TIMEOUT, ca_certs=None,
            verify_ssl=False, assert_hostname=None):

    class ValidHTTPSConnection(httplib.HTTPConnection):
        default_port = httplib.HTTPS_PORT

        def __init__(self, *args, **kwargs):
            httplib.HTTPConnection.__init__(self, *args, **kwargs)

        def connect(self):
            sock = socket.create_connection(
                address=(self.host, self.port),
                timeout=self.timeout,
            )
            if self._tunnel_host:
                self.sock = sock
                self._tunnel()

            self.sock = ssl.wrap_socket(
                sock, ca_certs=ca_certs, cert_reqs=ssl.CERT_REQUIRED)

            if assert_hostname is not None:
                match_hostname(self.sock.getpeercert(),
                               self.assert_hostname or self.host)

    class ValidHTTPSHandler(urllib2.HTTPSHandler):
        def https_open(self, req):
            return self.do_open(ValidHTTPSConnection, req)

    if verify_ssl:
        handlers = [ValidHTTPSHandler]
    else:
        try:
            handlers = [urllib2.HTTPSHandler(
                context=ssl._create_unverified_context())]
        except AttributeError:
            handlers = []

    opener = urllib2.build_opener(*handlers)

    if sys.version_info < (2, 6):
        default_timeout = socket.getdefaulttimeout()
        socket.setdefaulttimeout(timeout)
        try:
            return opener.open(url, data)
        finally:
            socket.setdefaulttimeout(default_timeout)
    return opener.open(url, data, timeout)



import pytest
from raven.transport.http import HTTPTransport


class MyException(Exception):
    pass


def test_decorator_exception(lambda_env, mock_client, lambda_event, lambda_context):

    client = mock_client()

    @client.capture_exceptions
    def test_func(event, context):
        raise MyException('There was an error.')

    with pytest.raises(MyException):
        test_func(event=lambda_event(), context=lambda_context(function_name='test_func'))

    assert client.events
    assert isinstance(client.remote.get_transport(), HTTPTransport)
    assert 'user' in client.events[0].keys()
    assert 'request' in client.events[0].keys()


def test_decorator_with_args(lambda_env, mock_client, lambda_event, lambda_context):
    client = mock_client()

    @client.capture_exceptions((MyException,))
    def test_func(event, context):
        raise Exception

    with pytest.raises(Exception):
        test_func(event=lambda_event(), context=lambda_context(function_name='test_func'))

    assert not client.events

    @client.capture_exceptions((MyException,))
    def test_func(event, context):
        raise MyException

    with pytest.raises(Exception):
        test_func(event=lambda_event(), context=lambda_context(function_name='test_func'))

    assert client.events


def test_decorator_without_exceptions(lambda_env, mock_client, lambda_event, lambda_context):
    client = mock_client()

    @client.capture_exceptions((MyException,))
    def test_func(event, context):
        return 0

    assert test_func(event=lambda_event(), context=lambda_context(function_name='test_func')) == 0


def test_decorator_without_kwargs(lambda_env, mock_client, lambda_event, lambda_context):

    client = mock_client()

    @client.capture_exceptions((MyException,))
    def test_func(event, context):
        raise MyException

    with pytest.raises(Exception):
        test_func(lambda_event(), lambda_context(function_name='test_func'))

    assert client.events




import os

# ---- SENTRY CONFIG ---- #
SENTRY_CLIENT = 'tests.contrib.django.tests.MockClient'
DISABLE_SENTRY_INSTRUMENTATION = True
SENTRY_ALLOW_ORIGIN = '*'

# ---- GENERIC DJANGO SETTINGS ---- #
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

SECRET_KEY = "Change this!"
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',

    'raven.contrib.django',
    'tests.contrib.django',
]


DATABASE_ENGINE='sqlite3'
DATABASES = {
    'default': {
        'NAME': ':memory:',
        'ENGINE': 'django.db.backends.sqlite3',
        'TEST_NAME': ':memory:',
    },
}
DATABASE_NAME = ':memory:'
TEST_DATABASE_NAME = ':memory:'
ROOT_URLCONF = 'tests.contrib.django.urls'
DEBUG = False
SITE_ID = 1

TEMPLATE_DEBUG = True
LANGUAGE_CODE = 'en'
LANGUAGES = (('en', 'English'),)
TEMPLATE_DIRS = [
    os.path.join(PROJECT_ROOT, 'templates'),
]
TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'APP_DIRS': True,
    'DIRS': TEMPLATE_DIRS,
}]
ALLOWED_HOSTS = ['*']

# ---- CELERY SETTINGS ---- #

try:
    import djcelery  # NOQA
    INSTALLED_APPS.append('djcelery')
except ImportError:
    pass

BROKER_HOST = "localhost"
BROKER_PORT = 5672
BROKER_USER = "guest"
BROKER_PASSWORD = "guest"
BROKER_VHOST = "/"
CELERY_ALWAYS_EAGER = True



import threading
from raven.utils.testutils import TestCase

from raven.base import Client
from raven.context import Context


class ContextTest(TestCase):
    def test_simple(self):
        context = Context()
        context.merge({'foo': 'bar'})
        context.merge({'biz': 'baz'})
        context.merge({'biz': 'boz'})
        assert context.get() == {
            'foo': 'bar',
            'biz': 'boz',
        }

    def test_tags(self):
        context = Context()
        context.merge({'tags': {'foo': 'bar'}})
        context.merge({'tags': {'biz': 'baz'}})
        assert context.get() == {
            'tags': {
                'foo': 'bar',
                'biz': 'baz',
            }
        }

    def test_extra(self):
        context = Context()
        context.merge({'extra': {'foo': 'bar'}})
        context.merge({'extra': {'biz': 'baz'}})
        assert context.get() == {
            'extra': {
                'foo': 'bar',
                'biz': 'baz',
            }
        }

    def test_thread_binding(self):
        client = Client()
        called = []

        class TestContext(Context):

            def activate(self):
                Context.activate(self)
                called.append('activate')

            def deactivate(self):
                called.append('deactivate')
                Context.deactivate(self)

        # The main thread activates the context but clear does not
        # deactivate.
        context = TestContext(client)
        context.clear()
        assert called == ['activate']

        # But another thread does.
        del called[:]

        def test_thread():
            # This activate is unnecessary as the first activate happens
            # automatically
            context.activate()
            context.clear()
        t = threading.Thread(target=test_thread)
        t.start()
        t.join()
        assert called == ['activate', 'activate', 'deactivate']



from __future__ import absolute_import

import os.path
import pytest
import sys

collect_ignore = [
    'tests/contrib/awslambda'
]

if sys.version_info[0] > 2:
    if sys.version_info[1] < 3:
        collect_ignore.append('tests/contrib/flask')
    if sys.version_info[1] == 2:
        collect_ignore.append('tests/handlers/logbook')

try:
    import gevent  # NOQA
except ImportError:
    collect_ignore.append('tests/transport/gevent')

try:
    import web  # NOQA
except ImportError:
    collect_ignore.append('tests/contrib/webpy')

try:
    import django  # NOQA
except ImportError:
    django = None
    collect_ignore.append('tests/contrib/django')

try:
    import Sanic  # NOQA
except ImportError:
    collect_ignore.append('tests/contrib/sanic')

try:
    import tastypie  # NOQA
except ImportError:
    collect_ignore.append('tests/contrib/django/test_tastypie.py')


use_djcelery = True
try:
    import djcelery  # NOQA
    # INSTALLED_APPS.append('djcelery')
except ImportError:
    use_djcelery = False


def pytest_runtest_teardown(item):
    if django:
        from raven.contrib.django.models import client
        client.events = []


@pytest.fixture
def project_root():
    return os.path.dirname(os.path.abspath(__file__))


@pytest.fixture
def mytest_model():

    from tests.contrib.django.models import MyTestModel
    return MyTestModel


@pytest.fixture(scope='function', autouse=False)
def user_instance(request, admin_user):
    request.cls.user = admin_user


@pytest.fixture(autouse=True)
def has_git_requirements(request, project_root):
    if request.node.get_marker('has_git_requirements'):
        if not os.path.exists(os.path.join(project_root, '.git', 'refs', 'heads', 'master')):
            pytest.skip('skipped test as project is not a git repo')



import pytest

import django

from raven.contrib.django.resolver import RouteResolver

try:
    from django.conf.urls import url, include
except ImportError:
    # for Django version less than 1.4
    from django.conf.urls.defaults import url, include


@pytest.fixture
def route_resolver():
    return RouteResolver()


@pytest.fixture
def urlconf():
    if django.VERSION < (1, 9):
        included_url_conf = (
            url(r'^foo/bar/(?P<param>[\w]+)', lambda x: ''),
        ), '', ''
    else:
        included_url_conf = ((
            url(r'^foo/bar/(?P<param>[\w]+)', lambda x: ''),
        ), '')

    if django.VERSION >= (2, 0):
        from django.urls import path, re_path

        example_url_conf = (
            re_path(r'^api/(?P<project_id>[\w_-]+)/store/$', lambda x: ''),
            re_path(r'^report/', lambda x: ''),
            re_path(r'^example/', include(included_url_conf)),
            path('api/v2/<int:project_id>/store/', lambda x: '')
        )
        return example_url_conf



"""
raven.utils.compat
~~~~~~~~~~~~~~~~~~

:copyright: (c) 2010-2016 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.

Utilities for writing code that runs on Python 2 and 3
"""
# flake8: noqa

# Copyright (c) 2010-2013 Benjamin Peterson
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
from __future__ import absolute_import

import operator
import sys
import types


PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str


try:
    advance_iterator = next
except NameError:
    def advance_iterator(it):
        return it.next()
next = advance_iterator


try:
    callable = callable
except NameError:
    def callable(obj):
        return any("__call__" in klass.__dict__ for klass in type(obj).__mro__)

if PY3:
    Iterator = object
else:
    class Iterator(object):

        def next(self):
            return type(self).__next__(self)


if PY3:
    def iterkeys(d, **kw):
        return iter(d.keys(**kw))

    def itervalues(d, **kw):
        return iter(d.values(**kw))

    def iteritems(d, **kw):
        return iter(d.items(**kw))

    def iterlists(d, **kw):
        return iter(d.lists(**kw))
else:
    def iterkeys(d, **kw):
        return d.iterkeys(**kw)

    def itervalues(d, **kw):
        return d.itervalues(**kw)

    def iteritems(d, **kw):
        return d.iteritems(**kw)

    def iterlists(d, **kw):
        return d.iterlists(**kw)


if PY3:
    def b(s):
        return s.encode("latin-1")

    def u(s):
        return s
    import io
    StringIO = io.StringIO
    BytesIO = io.BytesIO
else:
    def b(s):
        return s
    # Workaround for standalone backslash

    def u(s):
        return unicode(s.replace(r'\\', r'\\\\'), "unicode_escape")
    import StringIO
    StringIO = BytesIO = StringIO.StringIO


if PY3:
    exec_ = getattr(__import__('builtins'), 'exec')

    def reraise(tp, value, tb=None):
        try:
            if value is None:
                value = tp()
            if value.__traceback__ is not tb:
                raise value.with_traceback(tb)
            raise value
        finally:
            value = None
            tb = None

else:
    def exec_(_code_, _globs_=None, _locs_=None):
        """Execute code in a namespace."""
        if _globs_ is None:
            frame = sys._getframe(1)
            _globs_ = frame.f_globals
            if _locs_ is None:
                _locs_ = frame.f_locals
            del frame
        elif _locs_ is None:
            _locs_ = _globs_
        exec("""exec _code_ in _globs_, _locs_""")

    exec_("""def reraise(tp, value, tb=None):
    try:
        raise tp, value, tb
    finally:
        tb = None
""")


if sys.version_info[:2] == (3, 2):
    exec_("""def raise_from(value, from_value):
    try:
        if from_value is None:
            raise value
        raise value from from_value
    finally:
        value = None
""")
elif sys.version_info[:2] > (3, 2):
    exec_("""def raise_from(value, from_value):
    try:
        raise value from from_value
    finally:
        value = None
""")
else:
    def raise_from(value, from_value):
        raise value


if PY3:
    from urllib.error import HTTPError
    from http import client as httplib
    import urllib.request as urllib2
    from queue import Queue
    from urllib.parse import quote as urllib_quote
    from urllib import parse as urlparse
else:
    from urllib2 import HTTPError
    import httplib
    import urllib2
    from Queue import Queue
    from urllib import quote as urllib_quote
    import urlparse


def get_code(func):
    rv = getattr(func, '__code__', getattr(func, 'func_code', None))
    if rv is None:
        raise TypeError('Could not get code from %r' % type(func).__name__)
    return rv


def check_threads():
    try:
        from uwsgi import opt
    except ImportError:
        return

    # When `threads` is passed in as a uwsgi option,
    # `enable-threads` is implied on.
    if 'threads' in opt:
        return

    if str(opt.get('enable-threads', '0')).lower() in ('false', 'off', 'no', '0'):
        from warnings import warn
        warn(Warning('We detected the use of uwsgi with disabled threads.  '
                     'This will cause issues with the transport you are '
                     'trying to use.  Please enable threading for uwsgi.  '
                     '(Enable the "enable-threads" flag).'))



# -*- coding: utf-8 -*-

import datetime
import uuid
from decimal import Decimal
from raven.utils.testutils import TestCase

from raven.utils import json


class JSONTest(TestCase):
    def test_uuid(self):
        res = uuid.uuid4()
        json.dumps(res) == '"%s"' % res.hex

    def test_datetime(self):
        res = datetime.datetime(day=1, month=1, year=2011, hour=1, minute=1, second=1)
        assert json.dumps(res) == '"2011-01-01T01:01:01Z"'

    def test_set(self):
        res = set(['foo', 'bar'])
        assert json.dumps(res) in ('["foo", "bar"]', '["bar", "foo"]')

    def test_frozenset(self):
        res = frozenset(['foo', 'bar'])
        assert json.dumps(res) in ('["foo", "bar"]', '["bar", "foo"]')

    def test_unknown_type(self):

        class Unknown(object):
            def __repr__(self):
                return 'Unknown object'

        obj = Unknown()
        assert json.dumps(obj) == '"Unknown object"'

    def test_unknown_type_with_repr_error(self):

        class Unknown(object):
            def __repr__(self):
                raise Exception

        obj = Unknown()
        s = json.dumps(obj)
        assert isinstance(s, str)
        assert 'Unknown object at 0x' in s

    def test_decimal(self):
        d = {'decimal': Decimal('123.45')}
        assert json.dumps(d) == '{"decimal": "Decimal(\'123.45\')"}'



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, with_statement

import datetime
import django
import logging
import mock
import pytest
import re
import sys

from django.conf import settings
from django.core.exceptions import SuspiciousOperation
from django.core.signals import got_request_exception
from django.core.handlers.wsgi import WSGIRequest
from django.http import QueryDict
from django.template import TemplateSyntaxError
from django.test import TestCase
from django.test.client import Client as DjangoTestClient, ClientHandler as DjangoTestClientHandler
from django.utils.translation import gettext_lazy
from exam import fixture

try:
    from django.urls import reverse
except ImportError:
    # For Django version less than 1.10.
    from django.core.urlresolvers import reverse

from raven.base import Client
from raven.utils.compat import StringIO, iteritems, PY2, string_types, text_type
from raven.contrib.django.client import DjangoClient, record_sql
from raven.contrib.django.celery import CeleryClient
from raven.contrib.django.handlers import SentryHandler
from raven.contrib.django.models import (
    SentryDjangoHandler, client, get_client
)
from raven.contrib.django.middleware.wsgi import Sentry
from raven.contrib.django.templatetags.raven import sentry_public_dsn
from raven.contrib.django.views import is_valid_origin
from raven.transport import HTTPTransport
from raven.utils.serializer import transform

from .views import AppError

#from .models import MyTestModel

DJANGO_15 = django.VERSION >= (1, 5, 0)
DJANGO_18 = django.VERSION >= (1, 8, 0)
DJANGO_110 = django.VERSION >= (1, 10, 0)

MIDDLEWARE_ATTR = 'MIDDLEWARE' if DJANGO_110 else 'MIDDLEWARE_CLASSES'


def make_request():
    return WSGIRequest(environ={
        'wsgi.input': StringIO(),
        'REQUEST_METHOD': 'POST',
        'SERVER_NAME': 'testserver',
        'SERVER_PORT': '80',
        'CONTENT_TYPE': 'text/html',
        'ACCEPT': 'text/html',
    })


class MockClientHandler(DjangoTestClientHandler):
    def __call__(self, environ, start_response=[]):
        # this pretends doesn't require start_response
        return super(MockClientHandler, self).__call__(environ)


class MockSentryMiddleware(Sentry):
    def __call__(self, environ, start_response=[]):
        # this pretends doesn't require start_response
        return list(super(MockSentryMiddleware, self).__call__(environ, start_response))


class MockClient(DjangoClient):
    def __init__(self, *args, **kwargs):
        self.events = []
        super(MockClient, self).__init__(*args, **kwargs)

    def send(self, **kwargs):
        self.events.append(kwargs)

    def is_enabled(self, **kwargs):
        return True


class DisabledMockClient(MockClient):
    def is_enabled(self, **kwargs):
        return False


class Settings(object):
    """
    Allows you to define settings that are required for this function to work.

    >>> with Settings(SENTRY_LOGIN_URL='foo'): #doctest: +SKIP
    >>>     print settings.SENTRY_LOGIN_URL #doctest: +SKIP
    """

    NotDefined = object()

    def __init__(self, **overrides):
        self.overrides = overrides
        self._orig = {}

    def __enter__(self):
        for k, v in iteritems(self.overrides):
            self._orig[k] = getattr(settings, k, self.NotDefined)
            setattr(settings, k, v)

    def __exit__(self, exc_type, exc_value, traceback):
        for k, v in iteritems(self._orig):
            if v is self.NotDefined:
                delattr(settings, k)
            else:
                setattr(settings, k, v)


class ClientProxyTest(TestCase):
    def test_proxy_responds_as_client(self):
        assert get_client() == client

    @mock.patch.object(MockClient, 'captureMessage')
    def test_basic(self, captureMessage):
        client.captureMessage(message='foo')
        captureMessage.assert_called_once_with(message='foo')


@pytest.mark.usefixtures("user_instance")
class DjangoClientTest(TestCase):
    # Fixture setup/teardown
    urls = 'tests.contrib.django.urls'

    def setUp(self):
        self.raven = get_client()
        self.handler = SentryDjangoHandler(self.raven)
        self.handler.install()
        self.addCleanup(self.handler.uninstall)

    def test_basic(self):
        self.raven.captureMessage(message='foo')
        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert 'sentry.interfaces.Message' in event
        message = event['sentry.interfaces.Message']
        assert message['message'] == 'foo'
        assert event['level'] == logging.ERROR
        assert event['message'] == 'foo'
        assert isinstance(event['timestamp'], datetime.datetime)

    def test_signal_integration(self):
        try:
            int(None)
        except Exception:
            got_request_exception.send(sender=self.__class__, request=None)
        else:
            self.fail('Expected an exception.')

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert 'exception' in event
        exc = event['exception']['values'][-1]
        assert exc['type'] == 'TypeError'
        assert exc['value'], "int() argument must be a string or a number == not 'NoneType'"
        assert event['level'] == logging.ERROR
        assert event['message'], "TypeError: int() argument must be a string or a number == not 'NoneType'"

    @pytest.mark.skipif(sys.version_info[:2] == (2, 6), reason='Python 2.6')
    def test_view_exception(self):
        path = reverse('sentry-raise-exc')
        self.assertRaises(Exception, self.client.get, path)

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert 'exception' in event
        exc = event['exception']['values'][-1]
        assert exc['type'] == 'Exception'
        assert exc['value'] == 'view exception'
        assert event['level'] == logging.ERROR
        assert event['message'] == 'Exception: view exception'
        assert 'request' in event
        assert event['request']['url'] == 'http://testserver{}'.format(path)

    def test_request_data_unavailable_if_request_is_read(self):
        with Settings(**{MIDDLEWARE_ATTR: []}):
            path = reverse('sentry-readrequest-raise-exc')
            self.assertRaises(
                AppError,
                self.client.post,
                path,
                '{"a":"b"}',
                content_type='application/json')
        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert event['request']['data'] == '<unavailable>'

    def test_djangorestframeworkcompatmiddleware_fills_request_data(self):
        with Settings(**{MIDDLEWARE_ATTR: [
            'raven.contrib.django.middleware.DjangoRestFrameworkCompatMiddleware']}):
            path = reverse('sentry-readrequest-raise-exc')
            self.assertRaises(
                AppError,
                self.client.post,
                path,
                '{"a":"b"}',
                content_type='application/json')
        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert event['request']['data'] == '{"a":"b"}'

    def test_capture_event_with_request_middleware(self):
        path = reverse('sentry-trigger-event')
        resp = self.client.get(path)
        assert resp.status_code == 200

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert event['message'] == 'test'
        assert 'request' in event
        assert event['request']['url'] == 'http://testserver{}'.format(path)

    def test_user_info(self):
        with Settings(**{MIDDLEWARE_ATTR: [
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware']}):

            self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)
            assert 'user' in event

            user_info = event['user']
            assert user_info == {'ip_address': '127.0.0.1'}

            assert self.client.login(username='admin', password='password')

            self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)
            assert 'user' in event
            user_info = event['user']
            assert user_info == {
                'ip_address': '127.0.0.1',
                'username': self.user.username,
                'id': self.user.id,
                'email': self.user.email,
            }

    @pytest.mark.skipif(not DJANGO_15, reason='< Django 1.5')
    def test_get_user_info_abstract_user(self):

        from django.db import models
        from django.http import HttpRequest
        from django.contrib.auth.models import AbstractBaseUser

        class MyUser(AbstractBaseUser):
            USERNAME_FIELD = 'username'

            username = models.CharField(max_length=32)
            email = models.EmailField()

        user = MyUser(
            username='admin',
            email='admin@example.com',
            id=1,
        )

        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '127.0.0.1'
        request.user = user
        user_info = self.raven.get_user_info(request)
        assert user_info == {
            'ip_address': '127.0.0.1',
            'username': user.username,
            'id': user.id,
            'email': user.email,
        }

        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '127.0.0.1'
        request.META['HTTP_X_FORWARDED_FOR'] = '1.1.1.1, 2.2.2.2'
        request.user = user
        user_info = self.raven.get_user_info(request)
        assert user_info == {
            'ip_address': '1.1.1.1',
            'username': user.username,
            'id': user.id,
            'email': user.email,
        }

    @pytest.mark.skipif(not DJANGO_110, reason='< Django 1.10')
    def test_get_user_info_is_authenticated_property(self):
        from django.db import models
        from django.http import HttpRequest
        from django.contrib.auth.models import AbstractBaseUser

        class MyUser(AbstractBaseUser):
            USERNAME_FIELD = 'username'
            username = models.CharField(max_length=32)
            email = models.EmailField()

            @property
            def is_authenticated(self):
                return True

        user = MyUser(
            username='admin',
            email='admin@example.com',
            id=1,
        )

        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '127.0.0.1'
        request.user = user
        user_info = self.raven.get_user_info(request)
        assert user_info == {
            'ip_address': '127.0.0.1',
            'username': user.username,
            'id': user.id,
            'email': user.email,
        }

        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '127.0.0.1'
        request.META['HTTP_X_FORWARDED_FOR'] = '1.1.1.1, 2.2.2.2'
        request.user = user
        user_info = self.raven.get_user_info(request)
        assert user_info == {
            'ip_address': '1.1.1.1',
            'username': user.username,
            'id': user.id,
            'email': user.email,
        }

    def test_request_middleware_exception(self):
        with Settings(**{MIDDLEWARE_ATTR: ['tests.contrib.django.middleware.BrokenRequestMiddleware']}):
            self.assertRaises(ImportError, self.client.get, reverse('sentry-raise-exc'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)

            assert 'exception' in event
            exc = event['exception']['values'][-1]
            assert exc['type'] == 'ImportError'
            assert exc['value'] == 'request'
            assert event['level'] == logging.ERROR
            assert event['message'] == 'ImportError: request'

    def test_response_middlware_exception(self):
        if django.VERSION[:2] < (1, 3):
            return
        with Settings(**{MIDDLEWARE_ATTR: ['tests.contrib.django.middleware.BrokenResponseMiddleware']}):
            self.assertRaises(ImportError, self.client.get, reverse('sentry-no-error'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)

            assert 'exception' in event
            exc = event['exception']['values'][-1]
            assert exc['type'] == 'ImportError'
            assert exc['value'] == 'response'
            assert event['level'] == logging.ERROR
            assert event['message'] == 'ImportError: response'

    def test_broken_500_handler_with_middleware(self):
        with Settings(BREAK_THAT_500=True, INSTALLED_APPS=['raven.contrib.django']):
            client = DjangoTestClient(REMOTE_ADDR='127.0.0.1')
            client.handler = MockSentryMiddleware(MockClientHandler())

            self.assertRaises(Exception, client.get, reverse('sentry-raise-exc'))
            assert len(self.raven.events) == 2 or 4  # TODO: ash remove duplicate client events
            event = self.raven.events.pop(0)

            assert 'exception' in event
            exc = event['exception']['values'][-1]
            assert exc['type'] == 'Exception'
            assert exc['value'] == 'view exception'
            assert event['level'] == logging.ERROR
            assert event['message'] == 'Exception: view exception'

            event = self.raven.events.pop(0)

            assert 'exception' in event
            exc = event['exception']['values'][-1]
            assert exc['type'] == 'ValueError'
            assert exc['value'] == 'handler500'
            assert event['level'] == logging.ERROR
            assert event['message'] == 'ValueError: handler500'

    def test_view_middleware_exception(self):
        with Settings(**{MIDDLEWARE_ATTR: ['tests.contrib.django.middleware.BrokenViewMiddleware']}):
            self.assertRaises(ImportError, self.client.get, reverse('sentry-raise-exc'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)

            assert 'exception' in event
            exc = event['exception']['values'][-1]
            assert exc['type'] == 'ImportError'
            assert exc['value'] == 'view'
            assert event['level'] == logging.ERROR
            assert event['message'] == 'ImportError: view'

    @pytest.mark.skipif(DJANGO_18, reason='Django 1.8+ not supported')
    def test_template_name_as_view(self):
        self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        assert event['culprit'] == 'error.html'

    # def test_request_in_logging(self):
    #     resp = self.client.get(reverse('sentry-log-request-exc'))
    #     assert resp.status_code == 200

    #     assert len(self.raven.events) == 1
    #     event = self.raven.events.pop(0)

    #     assert event['culprit'] == 'tests.contrib.django.views in logging_request_exc'
    #     assert event['data']['META']['REMOTE_ADDR'] == '127.0.0.1'

    # TODO: Python bug #10805
    @pytest.mark.skipif(not PY2, reason='Python 2')
    def test_record_none_exc_info(self):
        # sys.exc_info can return (None, None, None) if no exception is being
        # handled anywhere on the stack. See:
        #  http://docs.python.org/library/sys.html#sys.exc_info
        record = logging.LogRecord(
            'foo',
            logging.INFO,
            pathname=None,
            lineno=None,
            msg='test',
            args=(),
            exc_info=(None, None, None),
        )
        handler = SentryHandler()
        handler.emit(record)

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert event['message'] == 'test'

    def test_404_middleware(self):
        with Settings(**{MIDDLEWARE_ATTR: ['raven.contrib.django.middleware.Sentry404CatchMiddleware']}):
            resp = self.client.get('/non-existent-page')
            assert resp.status_code == 404

            assert len(self.raven.events) == 1, [e['message'] for e in self.raven.events]
            event = self.raven.events.pop(0)

            assert event['level'] == logging.INFO
            assert event['logger'] == 'http404'

            assert 'request' in event
            http = event['request']
            assert http['url'] == 'http://testserver/non-existent-page'
            assert http['method'] == 'GET'
            assert http['query_string'] == ''
            assert http['data'] is None

    def test_404_middleware_when_disabled(self):
        extra_settings = {
            MIDDLEWARE_ATTR: ['raven.contrib.django.middleware.Sentry404CatchMiddleware'],
            'SENTRY_CLIENT': 'tests.contrib.django.tests.DisabledMockClient',
        }
        with Settings(**extra_settings):
            resp = self.client.get('/non-existent-page')
            assert resp.status_code == 404
            assert self.raven.events == []

    def test_invalid_client(self):
        extra_settings = {
            'SENTRY_CLIENT': 'raven.contrib.django.DjangoClient',  # default
        }
        # Should return fallback client (MockClient)
        client = get_client('nonexistent.and.invalid')

        # client should be valid, and the same as with the next call.
        assert client is get_client()

        with Settings(**extra_settings):
            assert isinstance(get_client(), DjangoClient)

    def test_transport_specification(self):
        extra_settings = {
            'SENTRY_TRANSPORT': 'raven.transport.HTTPTransport',
            'SENTRY_DSN': 'http://public:secret@example.com/1',
        }
        with Settings(**extra_settings):
            client = get_client(reset=True)
            assert type(client.remote.get_transport()) is HTTPTransport

    def test_response_error_id_middleware(self):
        # TODO: test with 500s
        with Settings(**{MIDDLEWARE_ATTR: [
                'raven.contrib.django.middleware.SentryResponseErrorIdMiddleware',
                'raven.contrib.django.middleware.Sentry404CatchMiddleware']}):
            resp = self.client.get('/non-existent-page')
            assert resp.status_code == 404
            headers = dict(resp.items())
            assert 'X-Sentry-ID' in headers
            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)
            assert event['event_id'] == headers['X-Sentry-ID']

    def test_get_client(self):
        assert get_client() == get_client()
        assert get_client('raven.base.Client').__class__ == Client
        assert get_client() == self.raven

        assert get_client('%s.%s' % (type(self.raven).__module__, type(self.raven).__name__)) == self.raven
        assert get_client() == self.raven

    def test_raw_post_data_partial_read(self):
        v = '{"foo": "bar"}'
        request = make_request()
        request.environ.update({
            'wsgi.input': StringIO(v + '\r\n\r\n'),
            'CONTENT_TYPE': 'application/octet-stream',
            'CONTENT_LENGTH': len(v),
        })
        request.read(1)

        self.raven.captureMessage(message='foo', request=request)

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        assert 'request' in event
        http = event['request']
        assert http['method'] == 'POST'
        assert http['data'] == '<unavailable>'

    def test_read_post_data(self):
        request = make_request()
        request.POST = QueryDict("foo=bar&ham=spam")
        request.read(1)

        self.raven.captureMessage(message='foo', request=request)

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        assert 'request' in event
        http = event['request']
        assert http['method'] == 'POST'
        assert http['data'] == {'foo': 'bar', 'ham': 'spam'}

    def test_request_capture(self):
        request = make_request()
        request.read(1)

        self.raven.captureMessage(message='foo', request=request)

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        assert 'request' in event
        http = event['request']
        assert http['method'] == 'POST'
        assert http['data'] == '<unavailable>'
        assert 'headers' in http
        headers = http['headers']
        assert 'Content-Type' in headers, headers.keys()
        assert headers['Content-Type'] == 'text/html'
        env = http['env']
        assert 'SERVER_NAME' in env, env.keys()
        assert env['SERVER_NAME'] == 'testserver'
        assert 'SERVER_PORT' in env, env.keys()
        assert env['SERVER_PORT'] == '80'

    def test_marks_django_frames_correctly(self):
        self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        frames = event['exception']['values'][-1]['stacktrace']['frames']
        for frame in frames:
            if frame['module'].startswith('django.'):
                assert frame.get('in_app') is False

    def test_adds_site_to_tags(self):
        self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        tags = event['tags']
        assert 'site' in event['tags']
        assert tags['site'] == 'example.com'

    def test_adds_site_to_tags_fallback(self):
        with Settings(SITE_ID=12345):  # nonexistent site, should fallback to SITE_ID
            self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))

            assert len(self.raven.events) == 1
            event = self.raven.events.pop(0)

            tags = event['tags']
            assert 'site' in event['tags']
            assert tags['site'] == 12345

    def test_settings_site_overrides_contrib(self):
        self.raven.site = 'FOO'
        self.assertRaises(TemplateSyntaxError, self.client.get, reverse('sentry-template-exc'))

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)

        tags = event['tags']
        assert 'site' in event['tags']
        assert tags['site'] == 'FOO'

    @mock.patch.object(WSGIRequest, 'build_absolute_uri')
    def test_suspicious_operation_in_build_absolute_uri(self, build_absolute_uri):
        build_absolute_uri.side_effect = SuspiciousOperation()
        request = make_request()
        request.META['HTTP_HOST'] = 'example.com'
        result = self.raven.get_data_from_request(request)
        build_absolute_uri.assert_called_once_with()
        assert 'request' in result
        assert result['request']['url'] == 'http://example.com/'


class DjangoTemplateTagTest(TestCase):
    @mock.patch('raven.contrib.django.DjangoClient.get_public_dsn')
    def test_sentry_public_dsn_no_args(self, get_public_dsn):
        sentry_public_dsn()
        get_public_dsn.assert_called_once_with(None)

    @mock.patch('raven.contrib.django.DjangoClient.get_public_dsn')
    def test_sentry_public_dsn_with_https(self, get_public_dsn):
        sentry_public_dsn('https')
        get_public_dsn.assert_called_once_with('https')


class DjangoLoggingTest(TestCase):
    def setUp(self):
        self.logger = logging.getLogger(__name__)
        self.raven = get_client()

    def test_request_kwarg(self):
        handler = SentryHandler()

        logger = self.logger
        logger.handlers = []
        logger.addHandler(handler)

        logger.error('This is a test error', extra={
            'request': WSGIRequest(environ={
                'wsgi.input': StringIO(),
                'REQUEST_METHOD': 'POST',
                'SERVER_NAME': 'testserver',
                'SERVER_PORT': '80',
                'CONTENT_TYPE': 'application/octet-stream',
                'ACCEPT': 'application/json',
            })
        })

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert 'request' in event
        http = event['request']
        assert http['method'] == 'POST'

    def test_tags(self):
        tags = {'tag1': 'test'}
        handler = SentryHandler(tags=tags)

        logger = self.logger
        logger.handlers = []
        logger.addHandler(handler)

        logger.error('This is a test error')

        assert len(self.raven.events) == 1
        event = self.raven.events.pop(0)
        assert 'tags' in event
        # event['tags'] also contains some other data, like 'site'
        assert 'tag1' in event['tags']
        assert event['tags']['tag1'] == tags['tag1']


class CeleryIsolatedClientTest(TestCase):
    def setUp(self):
        self.client = CeleryClient(
            dsn='sync+http://public:secret@example.com/1'
        )

    @mock.patch('raven.contrib.django.celery.send_raw')
    def test_send_encoded(self, send_raw):
        self.client.send_encoded('foo')

        send_raw.delay.assert_called_once_with('foo')

    @mock.patch('raven.contrib.django.celery.send_raw')
    def test_without_eager(self, send_raw):
        """
        Integration test to ensure it propagates all the way down
        and calls delay on the task.
        """
        self.client.captureMessage(message='test')

        assert send_raw.delay.call_count == 1


class IsValidOriginTestCase(TestCase):
    def test_setting_empty(self):
        with Settings(SENTRY_ALLOW_ORIGIN=None):
            assert not is_valid_origin('http://example.com')

    def test_setting_all(self):
        with Settings(SENTRY_ALLOW_ORIGIN='*'):
            assert is_valid_origin('http://example.com')

    def test_setting_uri(self):
        with Settings(SENTRY_ALLOW_ORIGIN=['http://example.com']):
            assert is_valid_origin('http://example.com')

    def test_setting_regexp(self):
        with Settings(SENTRY_ALLOW_ORIGIN=[re.compile('https?\://(.*\.)?example\.com')]):
            assert is_valid_origin('http://example.com')


class ReportViewTest(TestCase):
    urls = 'raven.contrib.django.urls'

    def setUp(self):
        super(ReportViewTest, self).setUp()
        self.path = reverse('raven-report', args=['1'], urlconf=self.urls)

    @mock.patch('raven.contrib.django.views.is_valid_origin')
    def test_calls_is_valid_origin_with_header(self, is_valid_origin):
        with self.settings(ROOT_URLCONF=self.urls):
            self.client.post(self.path, HTTP_ORIGIN='http://example.com')
        is_valid_origin.assert_called_once_with('http://example.com')

    @mock.patch('raven.contrib.django.views.is_valid_origin')
    def test_calls_is_valid_origin_with_header_as_get(self, is_valid_origin):
        with self.settings(ROOT_URLCONF=self.urls):
            self.client.get(self.path, HTTP_ORIGIN='http://example.com')
        is_valid_origin.assert_called_once_with('http://example.com')

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=False))
    def test_fails_on_invalid_origin(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com')
        assert resp.status_code == 403

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_options_call_sends_headers(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.options(self.path, HTTP_ORIGIN='http://example.com')
        assert resp.status_code == 200
        assert resp['Access-Control-Allow-Origin'] == 'http://example.com'
        assert resp['Access-Control-Allow-Methods'], 'GET, POST == OPTIONS'

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_missing_data(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com')
        assert resp.status_code == 400

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_invalid_data(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com',
                data='[1', content_type='application/octet-stream')
        assert resp.status_code == 400

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_sends_data(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com',
                data='{}', content_type='application/octet-stream')
        assert resp.status_code == 200
        event = client.events.pop(0)
        assert event == {'auth_header': None}

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_sends_authorization_header(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com',
                HTTP_AUTHORIZATION='Sentry foo/bar', data='{}', content_type='application/octet-stream')
        assert resp.status_code == 200
        event = client.events.pop(0)
        assert event == {'auth_header': 'Sentry foo/bar'}

    @mock.patch('raven.contrib.django.views.is_valid_origin', mock.Mock(return_value=True))
    def test_sends_x_sentry_auth_header(self):
        with self.settings(ROOT_URLCONF=self.urls):
            resp = self.client.post(self.path, HTTP_ORIGIN='http://example.com',
                HTTP_X_SENTRY_AUTH='Sentry foo/bar', data='{}',
                content_type='application/octet-stream')
        assert resp.status_code == 200
        event = client.events.pop(0)
        assert event == {'auth_header': 'Sentry foo/bar'}


class PromiseSerializerTestCase(TestCase):
    def test_basic(self):
        from django.utils.functional import lazy

        obj = lazy(lambda: 'bar', text_type)()
        res = transform(obj)
        expected = "'bar'" if not PY2 else "u'bar'"
        assert res == expected

    def test_handles_gettext_lazy(self):
        from django.utils.functional import lazy

        def fake_gettext(to_translate):
            return 'Igpay Atinlay'

        fake_gettext_lazy = lazy(fake_gettext, text_type)

        result = transform(fake_gettext_lazy("something"))
        assert isinstance(result, string_types)
        expected = "'Igpay Atinlay'" if not PY2 else "u'Igpay Atinlay'"
        assert result == expected

    def test_real_gettext_lazy(self):
        d = {text_type('lazy_translation'): gettext_lazy(text_type('testing'))}
        key = "'lazy_translation'" if not PY2 else "u'lazy_translation'"
        value = "'testing'" if not PY2 else "u'testing'"
        assert transform(d) == {key: value}


class ModelInstanceSerializerTestCase(object):
    def test_basic(self, mytest_model):
        instance = mytest_model()

        result = transform(instance)
        assert isinstance(result, string_types)
        assert result == '<MyTestModel: MyTestModel object>'


class QuerySetSerializerTestCase(object):
    def test_basic(self, mytest_model):
        from django.db.models.query import QuerySet
        obj = QuerySet(model=mytest_model)

        result = transform(obj)
        assert isinstance(result, string_types)
        assert result == '<QuerySet: model=MyTestModel>'


class SentryExceptionHandlerTest(TestCase):
    @fixture
    def request(self):
        return make_request()

    @fixture
    def exc_info(self):
        return (ValueError, ValueError('lol world'), None)

    def setUp(self):
        super(SentryExceptionHandlerTest, self).setUp()
        self.client = get_client()
        self.handler = SentryDjangoHandler(self.client)

    @mock.patch.object(MockClient, 'captureException')
    @mock.patch('sys.exc_info')
    def test_does_capture_exception(self, exc_info, captureException):
        exc_info.return_value = self.exc_info
        self.handler.exception_handler(request=self.request)

        captureException.assert_called_once_with(exc_info=self.exc_info, request=self.request)

    @mock.patch.object(MockClient, 'send')
    @mock.patch('sys.exc_info')
    def test_does_exclude_filtered_types(self, exc_info, mock_send):
        exc_info.return_value = self.exc_info
        try:
            self.client.ignore_exceptions = set(['ValueError'])

            self.handler.exception_handler(request=self.request)
        finally:
            self.client.ignore_exceptions.clear()

        assert not mock_send.called

    @mock.patch.object(MockClient, 'send')
    @mock.patch('sys.exc_info')
    def test_ignore_exceptions_with_expression_match(self, exc_info, mock_send):
        exc_info.return_value = self.exc_info

        try:
            if not PY2:
                self.client.ignore_exceptions = set(['builtins.*'])
            else:
                self.client.ignore_exceptions = set(['exceptions.*'])
            self.handler.exception_handler(request=self.request)
        finally:
            self.client.ignore_exceptions.clear()

        assert not mock_send.called

    @mock.patch.object(MockClient, 'send')
    @mock.patch('sys.exc_info')
    def test_ignore_exceptions_with_module_match(self, exc_info, mock_send):
        exc_info.return_value = self.exc_info

        try:
            if not PY2:
                self.client.ignore_exceptions = set(['builtins.ValueError'])
            else:
                self.client.ignore_exceptions = set(['exceptions.ValueError'])
            self.handler.exception_handler(request=self.request)
        finally:
            self.client.ignore_exceptions.clear()

        assert not mock_send.called


class SQLHookTestCase(TestCase):
    def test_wrong_params(self):
        query = 'SELECT COUNT(*) FROM mytestmodel WHERE id = %s'
        args = ['foobar', 42]
        record_sql(None, None, None, None, query, args)
        crumbs = get_client().context.breadcrumbs.get_buffer()
        self.assertEqual(crumbs[-1]['message'], query)



from __future__ import absolute_import

import django

from django.core.management import call_command
from django.test import TestCase
from mock import patch

from raven.contrib.django.models import client

DJANGO_18 = django.VERSION >= (1, 8, 0)


class RavenCommandTest(TestCase):
    @patch('raven.contrib.django.management.commands.raven.send_test_message')
    def test_basic(self, mock_send_test_message):
        call_command('raven', 'test')

        mock_send_test_message.assert_called_once_with(
            client, {
                'tags': None,
                'data': None,
            }
        )



from __future__ import absolute_import

import os.path
import pytest
import subprocess

from raven.utils.compat import string_types
from raven.versioning import fetch_git_sha, fetch_package_version


# Python 2.6 does not contain subprocess.check_output
def check_output(cmd, **kwargs):
    return subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        **kwargs
    ).communicate()[0]


@pytest.mark.has_git_requirements
def test_fetch_git_sha(project_root):
    result = fetch_git_sha(project_root)
    assert result is not None
    assert len(result) == 40
    assert isinstance(result, string_types)
    assert result == check_output(
        'git rev-parse --verify HEAD', shell=True, cwd=project_root
    ).decode('latin1').strip()


def test_fetch_package_version():
    result = fetch_package_version('raven')
    assert result is not None
    assert isinstance(result, string_types)



"""
raven.contrib.celery
~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2010-2012 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import

import logging

from celery.exceptions import SoftTimeLimitExceeded
from celery.signals import (
    after_setup_logger, task_failure, task_prerun, task_postrun
)
from raven.handlers.logging import SentryHandler


class CeleryFilter(logging.Filter):
    def filter(self, record):
        # Context is fixed in Celery 3.x so use internal flag instead
        extra_data = getattr(record, 'data', {})
        if not isinstance(extra_data, dict):
            return record.funcName != '_log_error'
        # Fallback to funcName for Celery 2.5
        return extra_data.get('internal', record.funcName != '_log_error')


def register_signal(client, ignore_expected=False):
    SentryCeleryHandler(client, ignore_expected=ignore_expected).install()


def register_logger_signal(client, logger=None, loglevel=logging.ERROR):
    filter_ = CeleryFilter()

    handler = SentryHandler(client)
    handler.setLevel(loglevel)
    handler.addFilter(filter_)

    def process_logger_event(sender, logger, loglevel, logfile, format,
                             colorize, **kw):
        # Attempt to find an existing SentryHandler, and if it exists ensure
        # that the CeleryFilter is installed.
        # If one is found, we do not attempt to install another one.
        for h in logger.handlers:
            if isinstance(h, SentryHandler):
                h.addFilter(filter_)
                return False

        logger.addHandler(handler)

    after_setup_logger.connect(process_logger_event, weak=False)


class SentryCeleryHandler(object):
    def __init__(self, client, ignore_expected=False):
        self.client = client
        self.ignore_expected = ignore_expected

    def install(self):
        task_prerun.connect(self.handle_task_prerun, weak=False)
        task_postrun.connect(self.handle_task_postrun, weak=False)
        task_failure.connect(self.process_failure_signal, weak=False)

    def uninstall(self):
        task_prerun.disconnect(self.handle_task_prerun)
        task_postrun.disconnect(self.handle_task_postrun)
        task_failure.disconnect(self.process_failure_signal)

    def process_failure_signal(self, sender, task_id, args, kwargs, einfo, **kw):
        if self.ignore_expected and hasattr(sender, 'throws') and isinstance(einfo.exception, sender.throws):
            return

        # This signal is fired inside the stack so let raven do its magic
        if isinstance(einfo.exception, SoftTimeLimitExceeded):
            fingerprint = ['celery', 'SoftTimeLimitExceeded', getattr(sender, 'name', sender)]
        else:
            fingerprint = None

        self.client.captureException(
            extra={
                'task_id': task_id,
                'task': sender,
                'args': args,
                'kwargs': kwargs,
            },
            fingerprint=fingerprint,
        )

    def handle_task_prerun(self, sender, task_id, task, **kw):
        self.client.context.activate()
        self.client.transaction.push(task.name)

    def handle_task_postrun(self, sender, task_id, task, **kw):
        self.client.transaction.pop(task.name)
        self.client.context.clear()



from __future__ import absolute_import

import celery
import logging

from raven.contrib.celery import (
    SentryCeleryHandler,
    register_logger_signal,
    CeleryFilter
)
from raven.handlers.logging import SentryHandler
from raven.utils.testutils import InMemoryClient, TestCase


class CeleryTestCase(TestCase):
    def setUp(self):
        super(CeleryTestCase, self).setUp()
        self.celery = celery.Celery(__name__)
        self.celery.conf.CELERY_ALWAYS_EAGER = True

        self.client = InMemoryClient()
        self.handler = SentryCeleryHandler(self.client, ignore_expected=True)
        self.handler.install()
        self.addCleanup(self.handler.uninstall)

    def test_simple(self):
        @self.celery.task(name='dummy_task')
        def dummy_task(x, y):
            return x / y

        dummy_task.delay(1, 2)
        dummy_task.delay(1, 0)
        assert len(self.client.events) == 1
        event = self.client.events[0]
        exception = event['exception']['values'][-1]
        assert event['transaction'] == 'dummy_task'
        assert exception['type'] == 'ZeroDivisionError'

    def test_ignore_expected(self):
        @self.celery.task(name='dummy_task', throws=(ZeroDivisionError,))
        def dummy_task(x, y):
            return x / y

        dummy_task.delay(1, 2)
        dummy_task.delay(1, 0)
        assert len(self.client.events) == 0


class CeleryLoggingHandlerTestCase(TestCase):
    def setUp(self):
        super(CeleryLoggingHandlerTestCase, self).setUp()

        self.client = InMemoryClient()

        # register the logger signal
        # and unregister the signal when the test is done
        register_logger_signal(self.client)
        receiver = celery.signals.after_setup_logger.receivers[0][1]
        self.addCleanup(celery.signals.after_setup_logger.disconnect, receiver)

        # remove any existing handlers and restore
        # them when complete
        self.root = logging.getLogger()
        for handler in self.root.handlers:
            self.root.removeHandler(handler)
            self.addCleanup(self.root.addHandler, handler)

    def test_handler_added(self):
        # Given: there are no handlers configured
        assert not self.root.handlers

        # When: the after_setup_logger signal is sent
        celery.signals.after_setup_logger.send(
            sender=None, logger=self.root,
            loglevel=logging.WARNING, logfile=None,
            format=u'', colorize=False,
        )

        # Then: there is 1 new handler
        assert len(self.root.handlers) == 1

        # Then: the new handler is an instance of
        # `raven.handlers.logging.SentryHandler`
        handler = self.root.handlers[0]
        assert isinstance(handler, SentryHandler)

        # Then: the handler has 1 filter
        assert len(handler.filters) == 1

        # Then: the filter is a CeleryFilter
        _filter = handler.filters[0]
        assert isinstance(_filter, CeleryFilter)

        # set up the handler to be removed once the test is done
        self.addCleanup(self.root.removeHandler, handler)

    def test_handler_updated(self):

        # Given: there is 1 preconfigured SentryHandler
        # with no filters
        handler = SentryHandler(self.client)
        assert not handler.filters
        self.root.addHandler(handler)
        # set up the handler to be removed once the test is done
        self.addCleanup(self.root.removeHandler, handler)

        # When: the after_setup_logger signal is sent
        celery.signals.after_setup_logger.send(
            sender=None, logger=self.root,
            loglevel=logging.WARNING, logfile=None,
            format=u'', colorize=False,
        )

        # Then: there is still just 1 handler
        assert len(self.root.handlers) == 1

        # Then: the existing handler is an instance of
        # `raven.handlers.logging.SentryHandler`
        handler = self.root.handlers[0]
        assert isinstance(handler, SentryHandler)

        # Then: the existing handler has 1 filter
        assert len(handler.filters) == 1

        # Then: the filter is a CeleryFilter
        _filter = handler.filters[0]
        assert isinstance(_filter, CeleryFilter)

    def test_subclassed_handler_updated(self):

        # Given: there is 1 preconfigured CustomHandler
        # with no filters
        class CustomHandler(SentryHandler):
            pass

        handler = CustomHandler(self.client)
        assert not handler.filters
        self.root.addHandler(handler)
        # set up the handler to be removed once the test is done
        self.addCleanup(self.root.removeHandler, handler)

        # When: the after_setup_logger signal is sent
        celery.signals.after_setup_logger.send(
            sender=None, logger=self.root,
            loglevel=logging.WARNING, logfile=None,
            format=u'', colorize=False,
        )

        # Then: there is still just 1 handler
        assert len(self.root.handlers) == 1

        # Then: the existing handler is an instance of
        # `CustomHandler`
        handler = self.root.handlers[0]
        assert isinstance(handler, CustomHandler)

        # Then: the existing handler has 1 filter
        assert len(handler.filters) == 1

        # Then: the filter is a CeleryFilter
        _filter = handler.filters[0]
        assert isinstance(_filter, CeleryFilter)
